@@ -9,6 +9,7 @@ from typing import List, Optional, Tuple
 import numpy as np
 
 from redvox.api900.reader import WrappedRedvoxPacket
+from redvox.api900.reader_utils import empty_array
 from redvox.api900.timesync import tri_message_stats
 from redvox.common import date_time_utils as dt
 from redvox.common import file_statistics as fh
@@ -19,9 +20,12 @@ class TimeSyncData:
     """
     Stores latencies, revised start times, time sync server offsets, and difference in adjusted machine time and
     acquisition server time
+    Note that inputs to this must be continuous data; any change in sensor mach_time_zero or sample_rate
+    requires a new TimeSyncData object
     ALL times in microseconds
     properties:
         rev_start_times: revised machine start times (per packet)
+        server_acquisition_times: time packet arrived at server (per packet)
         latencies: latencies (per packet)
         best_latency: the lowest latency among packets
         best_latency_index: index in latencies array that contains the best latency
@@ -31,21 +35,25 @@ class TimeSyncData:
         offset_stats: StatsContainer for offsets
         num_packets: the number of packets being analyzed
         sample_rate_hz: the sample rate in Hz of all packets
+        packet_duration: the duration of all packets
+        mach_time_zero: the start time of the app in machine time
         num_tri_messages: number of tri-message exchanges (per packet)
         tri_message_coeffs: list of 6-tuples containing the tri-message exchange coefficients (up to 1 tuple per packet)
+        best_tri_msg_indices: the index of the best tri-message for each of the tri_message_coeffs tuples
         acquire_travel_time: calculated time it took packet to reach server (up to 1 per packet)
         bad_packets: indices of packets with an undefined or 0 latency
     """
 
     def __init__(self, wrapped_packets: List[WrappedRedvoxPacket] = None):
         """
-        Initialize properties, assume all packets share the same sample rate
+        Initialize properties, assume all packets share the same sample rate and are continuous
         :param wrapped_packets: list of data packets
         """
         self.latency_stats: stats_helper.StatsContainer = stats_helper.StatsContainer("latency")
         self.offset_stats: stats_helper.StatsContainer = stats_helper.StatsContainer("offset")
-        if wrapped_packets is None:
+        if wrapped_packets is None or len(wrapped_packets) < 1:
             self.rev_start_times: np.ndarray = np.ndarray((0, 0))
+            self.server_acquisition_times: np.ndarray = np.ndarray((0, 0))
             self.latencies: np.ndarray = np.ndarray((0, 0))
             self.best_latency: Optional[float] = None
             self.best_latency_index: Optional[int] = None
@@ -53,8 +61,11 @@ class TimeSyncData:
             self.best_offset: float = 0.0
             self.num_packets: int = 0
             self.sample_rate_hz: Optional[float] = None
+            self.packet_duration: int = 0
+            self.mach_time_zero: Optional[float] = None
             self.num_tri_messages: np.ndarray = np.ndarray((0, 0))
             self.tri_message_coeffs: List[Tuple] = []
+            self.best_tri_msg_indices: List[int] = []
             self.acquire_travel_time: np.ndarray = np.ndarray((0, 0))
             self.bad_packets: List[int] = []
         else:
@@ -65,30 +76,39 @@ class TimeSyncData:
         Sets the time statistics between machine, time server, and acquisition server for the packets given.
         :param wrapped_packets: wrapped packets with same sample rate
         """
-        self.sample_rate_hz = wrapped_packets[0].microphone_sensor().sample_rate_hz()  # sample rate of all packets
-        self.rev_start_times = np.zeros(len(wrapped_packets))  # array of app start times
+        self.server_acquisition_times = np.zeros(len(wrapped_packets))  # array of server acquisition times
+        self.rev_start_times = np.zeros(len(wrapped_packets))   # array of app start times
         self.num_tri_messages = np.zeros(len(wrapped_packets))  # number of tri-message exchanges per packet
         self.latencies = np.zeros(len(wrapped_packets))  # array of minimum latencies
         self.offsets = np.zeros(len(wrapped_packets))  # array of offset applied to machine time to get sync time
         self.num_packets = len(wrapped_packets)  # number of packets in list
-        self.tri_message_coeffs = []  # a list of tri-message coefficients
-        self.bad_packets = []  # list of packets that contain invalid data
-        server_time = np.zeros(len(wrapped_packets))  # array of server acquisition times
+        self.tri_message_coeffs = []    # a list of tri-message coefficients
+        self.best_tri_msg_indices = []  # a list of the best latency index in each set of tri-message coefficients
+        self.bad_packets = []   # list of packets that contain invalid data
+        sample_rates = np.zeros(len(wrapped_packets))       # list of sample rates, should all be the same
+        mach_time_zeros = np.zeros(len(wrapped_packets))    # list of mach time zeros, should all the be same
 
         # get the server acquisition time, app start time, and tri message stats
         for i, wrapped_packet in enumerate(wrapped_packets):
-            server_time[i] = wrapped_packet.server_timestamp_epoch_microseconds_utc()
+            # pass the mach_time_zero and sample_rate into arrays
+            sample_rates[i] = wrapped_packet.microphone_sensor().sample_rate_hz()
+            mach_time_zeros[i] = wrapped_packet.mach_time_zero()
+            self.server_acquisition_times[i] = wrapped_packet.server_timestamp_epoch_microseconds_utc()
             self.rev_start_times[i] = wrapped_packet.app_file_start_timestamp_epoch_microseconds_utc()
             self._compute_tri_message_stats(wrapped_packet, i)
 
+        # check sample rate and mach time zero (if it exists) for changes.  if it changed, sync will not work.
+        if not self._validate_sensor_settings(sample_rates, mach_time_zeros):
+            raise Exception("ERROR: Sensor settings changed; separate data based on changes and re-analyze")
         self.find_bad_packets()
         self.evaluate_latencies_and_offsets()
         # set the packet duration
-        packet_duration = dt.seconds_to_microseconds(fh.get_duration_seconds_from_sample_rate(int(self.sample_rate_hz)))
+        self.packet_duration = dt.seconds_to_microseconds(fh.get_duration_seconds_from_sample_rate(
+            int(self.sample_rate_hz)))
         # apply duration to app start to get packet end time, then subtract
         # that from server acquire time to get travel time to acquisition server
         # ASSUMING that acquisition and time-sync server have the same time source
-        self.acquire_travel_time = server_time - (self.rev_start_times + packet_duration)
+        self.acquire_travel_time = self.server_acquisition_times - (self.rev_start_times + self.packet_duration)
 
     def evaluate_latencies_and_offsets(self):
         """
@@ -160,7 +180,7 @@ class TimeSyncData:
                                                     b1_coeffs,
                                                     b2_coeffs,
                                                     b3_coeffs)
-            # check if we actually have unique exchanges to evaluate
+            # check if we actually have exchanges to evaluate
             if self.num_tri_messages[index] > 0:
                 # Concatenate d1 and d3 arrays, and o1 and o3 arrays when passing values into stats class
                 #  note the last parameter multiplies the number of exchanges by 2, as there are two latencies
@@ -173,6 +193,8 @@ class TimeSyncData:
                                       self.num_tri_messages[index] * 2)
                 # set the best latency and offset based on the packet's metadata, or tri-message stats if no metadata
                 latency: Optional[float] = packet.best_latency()
+                # pass the location of the best calculated latency
+                self.best_tri_msg_indices.append(tms.best_latency_index)
                 if latency is None:
                     # no metadata
                     self.latencies[index] = tms.best_latency
@@ -184,15 +206,43 @@ class TimeSyncData:
                     self.offsets[index] = tms.best_offset
                 else:
                     self.offsets[index] = offset
-            else:
-                # there is a time sync channel and it's empty, so default to 0
-                self.latencies[index] = 0.0
-                self.offsets[index] = 0.0
-        # if there is no time sync channel (usually no communication between device and server), default to 0
-        else:
-            self.tri_message_coeffs.append(())
-            self.latencies[index] = 0.0
-            self.offsets[index] = 0.0
+                # everything has been computed, time to return
+                return
+        # If here, there are no exchanges to read or there is no sync channel.
+        # write default or empty values to the correct properties
+        self.latencies[index] = 0.0
+        self.offsets[index] = 0.0
+        self.tri_message_coeffs.append((empty_array(), empty_array(), empty_array(),
+                                        empty_array(), empty_array(), empty_array()))
+        # pass the location of the best latency
+        self.best_tri_msg_indices.append(-1)
+        # noinspection PyTypeChecker
+        self.latency_stats.add(0, 0, 0)
+        # noinspection PyTypeChecker
+        self.offset_stats.add(0, 0, 0)
+
+    def _validate_sensor_settings(self, sample_rates: np.array, mach_time_zeros: np.array) -> bool:
+        """
+        Examine all sample rates and mach time zeros to ensure that sensor settings do not change
+        Sets the sample rate and mach time zero if there is no change
+        :param sample_rates: sample rates of all packets from sensor
+        :param mach_time_zeros: machine time zero of all packets from sensor
+        :return: True if sensor settings do not change
+        """
+        # set the sample rate and mach time zero
+        self.sample_rate_hz = sample_rates[0]
+        self.mach_time_zero = mach_time_zeros[0]
+        if len(sample_rates) > 1:
+            # if there's more than 1 value, we need to compare them
+            for i in range(1, len(sample_rates)):
+                if self.sample_rate_hz != sample_rates[i]:
+                    print("ERROR: sample rate in data packets has changed.")
+                    return False
+                # process only non-nan mach time zeros
+                if not np.isnan(self.mach_time_zero) and self.mach_time_zero != mach_time_zeros[i]:
+                    print("ERROR: mach time zero in data packets has changed.")
+                    return False
+        return True
 
     def find_bad_packets(self):
         """
@@ -306,6 +356,36 @@ class TimeSyncData:
         return self.rev_start_times[self.best_latency_index]
 
 
+def validate_sensors(wrapped_packets: List[WrappedRedvoxPacket]) -> bool:
+    """
+    Examine all sample rates and mach time zeros to ensure that sensor settings do not change
+    :param wrapped_packets: a list of wrapped redvox packets to read
+    :return: True if sensor settings do not change
+    """
+    # check that we have packets to read
+    num_packets = len(wrapped_packets)
+    if num_packets < 1:
+        print("ERROR: no data to validate.")
+        return False
+    # if we have more than one packet, we need to validate the data
+    elif num_packets > 1:
+        sample_rates = np.zeros(num_packets)
+        mach_time_zeros = np.zeros(num_packets)
+        for j, wrapped_packet in enumerate(wrapped_packets):
+            sample_rates[j] = wrapped_packet.microphone_sensor().sample_rate_hz()
+            mach_time_zeros[j] = wrapped_packet.mach_time_zero()
+        for i in range(1, len(sample_rates)):
+            if sample_rates[0] != sample_rates[i]:
+                print("ERROR: sample rate in data packets has changed.")
+                return False
+            # process only non-nan mach time zeros
+            if not np.isnan(mach_time_zeros[0]) and mach_time_zeros[0] != mach_time_zeros[i]:
+                print("ERROR: mach time zero in data packets has changed.")
+                return False
+    # we get here if all packets have the same sample rate and mach time zero
+    return True
+
+
 def update_evenly_sampled_time_array(tsd: TimeSyncData, num_samples: float = None,
                                      time_start_array: np.array = None) -> np.ndarray:
     """
@@ -323,18 +403,17 @@ def update_evenly_sampled_time_array(tsd: TimeSyncData, num_samples: float = Non
     if time_start_array is None:
         # replace the time_start_array with values from tsd; convert tsd times to seconds
         time_start_array = tsd.rev_start_times / dt.MICROSECONDS_IN_SECOND
-
     num_files = tsd.num_packets
-    if num_samples is None:
-        num_samples = fh.get_num_points_from_sample_rate(tsd.sample_rate_hz)
-    t_dt = 1.0 / tsd.sample_rate_hz
-
     # the TimeSyncData must have the same number of packets as the number of elements in time_start_array
     if num_files != len(time_start_array):
         # alert the user, then quit
         raise Exception("ERROR: Attempted to update a time array that doesn't contain "
                         "the same number of elements as the TimeSyncData!")
 
+    if num_samples is None:
+        num_samples = fh.get_num_points_from_sample_rate(tsd.sample_rate_hz)
+    t_dt = 1.0 / tsd.sample_rate_hz
+
     # Use TimeSyncData object to find best start index.
     # Samples before will be the number of decoders before a0 times the number of samples in a file.
     # Samples after will be the number of decoders after a0 times the number of samples in a file minus 1;
@@ -417,6 +496,8 @@ def sync_packet_time_900(wrapped_packets_fs: list, verbose: bool = False):
 
     # convert NaN latencies to 0
     latencies = np.nan_to_num(latencies)
+    # convert any negative latency to 0.
+    latencies[latencies < 0] = 0
 
     if len(np.nonzero(latencies)[0]) == 0:  # if no decoders synced properly, all latencies are 0
         sync_server = wrapped_packets_fs[0].time_synchronization_server()
@@ -425,8 +506,6 @@ def sync_packet_time_900(wrapped_packets_fs: list, verbose: bool = False):
                 wrapped_packets_fs[0].redvox_id(), sync_server))
         offset = 0
     else:
-        # convert any negative latency to 0.
-        latencies[latencies < 0] = 0
         d1_min = np.min(latencies[latencies != 0])
         best_d1_index = np.where(latencies == d1_min)[0][0]
         # correct each packet's timestamps
