@@ -1,13 +1,17 @@
 """
 This module contains classes and enums for working with generic RedVox packet metadata through the cloud API.
 """
-import requests
+
 from dataclasses import dataclass
-from typing import Callable, Dict, List, Optional
+import json
+from typing import Callable, Dict, List, Optional, TypeVar, Tuple
 
 from dataclasses_json import dataclass_json
+import requests
+from redvox.api1000.wrapped_redvox_packet.wrapped_packet import WrappedRedvoxPacketM
 
-from redvox.cloud.api import ApiConfig, post_req
+from redvox.cloud.api import post_req
+from redvox.cloud.config import RedVoxConfig
 from redvox.cloud.routes import RoutesV1
 
 
@@ -17,6 +21,7 @@ class AudioMetadata:
     """
     Metadata associated with audio sensors.
     """
+
     sensor_name: Optional[str] = None
     sample_rate: Optional[float] = None
     first_sample_ts: Optional[int] = None
@@ -33,6 +38,7 @@ class SingleMetadata:
     Metadata associated with sensors that only contain a single dimensional channel of data
         (barometer, light, proximity).
     """
+
     sensor_name: Optional[str] = None
     timestamps_microseconds_utc_count: Optional[int] = None
     payload_count: Optional[int] = None
@@ -51,6 +57,7 @@ class XyzMetadata:
     """
     Metadata for sensors that have 3-dimensions of data (accelerometer, gyroscope, magenetometer)
     """
+
     sensor_name: Optional[str] = None
     timestamps_microseconds_utc_count: Optional[int] = None
     payload_count: Optional[int] = None
@@ -75,6 +82,7 @@ class LocationMetadata:
     """
     Metadata associated with the location sensor.
     """
+
     sensor_name: Optional[str] = None
     timestamps_microseconds_utc_count: Optional[int] = None
     payload_count: Optional[int] = None
@@ -105,6 +113,7 @@ class PacketMetadataResult:
     """
     Metadata associated with RedVox API 900 packets.
     """
+
     api: Optional[int] = None
     station_id: Optional[str] = None
     station_uuid: Optional[str] = None
@@ -142,10 +151,12 @@ class PacketMetadataResult:
 
 @dataclass_json
 @dataclass
+# pylint: disable=C0103
 class AvailableMetadata:
     """
     Contains definitions for all available metadata that an be requested from the cloud api.
     """
+
     Api: str = "Api"
     StationId: str = "StationId"
     StationUuid: str = "StationUuid"
@@ -229,6 +240,7 @@ class MetadataReq:
     """
     The definition of a metadata request. Fields should include the definitions defined by AvailableMetadata.
     """
+
     auth_token: str
     start_ts_s: int
     end_ts_s: int
@@ -243,15 +255,180 @@ class MetadataResp:
     """
     Response of a metadata request.
     """
+
     metadata: List[PacketMetadataResult]
 
 
+@dataclass_json
+@dataclass
+class StationStatusReq:
+    secret_token: Optional[str]
+    auth_token: str
+    start_ts_s: float
+    end_ts_s: float
+    station_ids: List[str]
+
+
+@dataclass_json
+@dataclass
+class StationStatus:
+    # API status
+    api: str
+    sub_api: Optional[str]
+
+    # Recording status
+    recording_state: str
+
+    # Timing status
+    app_start_timestamp: float
+    timestamp: float
+    synch_enabled: bool
+    synch_latency: Optional[float]
+
+    # Authentication status
+    station_id: str
+    station_uuid: str
+    authenticated_user: Optional[str]
+    private: bool
+
+    # Audio status
+    sampling_rate: float
+    bit_depth: Optional[float]
+    sensor_name: str
+    samples_per_packet: int
+
+    # Additional sensor status
+    additional_sensors: List[Tuple[str, str]]
+
+    # Movement status
+    movement_detected: Optional[bool]
+
+    # Location status
+    location_provider: Optional[str]
+    latitude: Optional[float]
+    longitude: Optional[float]
+    altitude: Optional[float]
+    speed: Optional[float]
+
+    # Station info
+    make: str
+    model: str
+    os: str
+    os_version: str
+    client_version: str
+    settings: Optional[Dict]
+
+    # Station metrics
+    network_type: Optional[str]
+    power_state: Optional[str]
+    screen_state: Optional[str]
+    cell_service_state: Optional[str]
+    battery: Optional[float]
+    temperature: Optional[float]
+    network_strength: Optional[float]
+
+    diffs: Optional[List[Dict]]
+
+
+@dataclass_json
+@dataclass
+class DataAvailability:
+    station_id: str
+    station_uuid: str
+    start: int
+    end: int
+    total_packets: int
+    expected_packets: int
+
+
+@dataclass_json
+@dataclass
+class StationStatusResp:
+    station_statuses: List[StationStatus]
+    data_availabilities: List[DataAvailability]
+
+
+# pylint: disable=C0103
+T = TypeVar("T")
+
+
+def _get(key: str, json_dict: Dict, default: Optional[T] = None) -> Optional[T]:
+    if key in json_dict:
+        return json_dict[key]
+
+    return default
+
+
+@dataclass
+class AdditionalMetadata:
+    """
+    Representation of additional metadata from database.
+    """
+
+    data_key: Optional[str]
+
+    @staticmethod
+    def from_dict(json_dict: Optional[Dict]) -> Optional["AdditionalMetadata"]:
+        """
+        Converts JSON into additional metadata.
+        :param json_dict: JSON to convert.
+        :return: Optional additional metadata instance.
+        """
+        if json_dict is None:
+            return None
+
+        return AdditionalMetadata(_get("data_key", json_dict))
+
+
+@dataclass
+class DbPacket:
+    """
+    Representation of a packet loaded from the database.
+    """
+
+    _id: Optional[str]
+    metadata: Optional[WrappedRedvoxPacketM]
+    additional_metadata: Optional[AdditionalMetadata]
+
+    @staticmethod
+    def from_dict(json_dict: Dict) -> "DbPacket":
+        """
+        Converts JSON to a DbPacket object.
+        :param json_dict: JSON to convert.
+        :return: An instance of a DbPacket.
+        """
+        return DbPacket(
+            _get("_id", json_dict),
+            WrappedRedvoxPacketM.from_json(json.dumps(json_dict["metadata"])),
+            AdditionalMetadata.from_dict(_get("additional_metadata", json_dict)),
+        )
+
+
+@dataclass
+class MetadataRespM:
+    """
+    A metadata response for API M.
+    """
+
+    db_packets: List[DbPacket]
+
+    @staticmethod
+    def from_json(json_dicts: Dict) -> "MetadataRespM":
+        """
+        Converts a JSON dictionary into a response.
+        :param json_dicts: Dictionary to use for conversion.
+        :return: The converted response.
+        """
+        return MetadataRespM(list(map(DbPacket.from_dict, json_dicts["db_packets"])))
+
+
 @dataclass_json
 @dataclass
 class TimingMetaRequest:
     """
     Request for timing metadata.
     """
+
     auth_token: str
     start_ts_s: int
     end_ts_s: int
@@ -265,6 +442,7 @@ class TimingMeta:
     """
     Timing metadata extracted from an individual packet.
     """
+
     station_id: str
     start_ts_os: float
     start_ts_mach: float
@@ -280,53 +458,116 @@ class TimingMetaResponse:
     """
     Response of obtaining timing metadta.
     """
+
     items: List[TimingMeta]
 
 
-def request_timing_metadata(api_config: ApiConfig,
-                            timing_req: TimingMetaRequest,
-                            session: Optional[requests.Session] = None,
-                            timeout: Optional[float] = None) -> TimingMetaResponse:
+def request_timing_metadata(
+    redvox_config: RedVoxConfig,
+    timing_req: TimingMetaRequest,
+    session: Optional[requests.Session] = None,
+    timeout: Optional[float] = None,
+) -> TimingMetaResponse:
     """
     Retrieve timing metadata.
-    :param api_config: An instance of the API configuration.
+    :param redvox_config: An instance of the API configuration.
     :param timing_req: An instance of a timing request.
     :param session: An (optional) session for re-using an HTTP client.
+    :param timeout: An (optional) timeout.
     :return: An instance of a timing response.
     """
 
     def handle_resp(resp) -> TimingMetaResponse:
-        json: List[Dict] = resp.json()
+        json_content: List[Dict] = resp.json()
         # noinspection Mypy
-        items: List[TimingMeta] = list(map(TimingMeta.from_dict, json))
+        # pylint: disable=E1101
+        items: List[TimingMeta] = list(map(TimingMeta.from_dict, json_content))
         return TimingMetaResponse(items)
 
-    res: Optional[TimingMetaResponse] = post_req(api_config,
-                                                 RoutesV1.TIMING_METADATA_REQ,
-                                                 timing_req,
-                                                 handle_resp,
-                                                 session,
-                                                 timeout)
+    res: Optional[TimingMetaResponse] = post_req(
+        redvox_config,
+        RoutesV1.TIMING_METADATA_REQ,
+        timing_req,
+        handle_resp,
+        session,
+        timeout,
+    )
 
     return res if res else TimingMetaResponse([])
 
 
-def request_metadata(api_config: ApiConfig,
-                     packet_metadata_req: MetadataReq,
-                     session: Optional[requests.Session] = None,
-                     timeout: Optional[float] = None) -> Optional[MetadataResp]:
+def request_metadata(
+    redvox_config: RedVoxConfig,
+    packet_metadata_req: MetadataReq,
+    session: Optional[requests.Session] = None,
+    timeout: Optional[float] = None,
+) -> Optional[MetadataResp]:
     """
     Requests generic metadata from the cloud API.
-    :param api_config: An instance of the API config.
+    :param redvox_config: An instance of the API config.
     :param packet_metadata_req: An instance of a metadata request.
     :param session: An (optional) session for re-using an HTTP client.
+    :param timeout: An (optional) timeout.
     :return: A metadata response on successful call or None if there is an error.
     """
     # noinspection Mypy
-    handle_resp: Callable[[requests.Response], MetadataResp] = lambda resp: MetadataResp.from_dict(resp.json())
-    return post_req(api_config,
-                    RoutesV1.METADATA_REQ,
-                    packet_metadata_req,
-                    handle_resp,
-                    session,
-                    timeout)
+    # pylint: disable=E1101
+    handle_resp: Callable[
+        [requests.Response], MetadataResp
+    ] = lambda resp: MetadataResp.from_dict(resp.json())
+    return post_req(
+        redvox_config,
+        RoutesV1.METADATA_REQ,
+        packet_metadata_req,
+        handle_resp,
+        session,
+        timeout,
+    )
+
+
+def request_metadata_m(
+    redvox_config: RedVoxConfig,
+    packet_metadata_req: MetadataReq,
+    session: Optional[requests.Session] = None,
+    timeout: Optional[float] = None,
+) -> Optional[MetadataRespM]:
+    """
+    Requests generic metadata from the cloud API.
+    :param redvox_config: An instance of the API config.
+    :param packet_metadata_req: An instance of a metadata request.
+    :param session: An (optional) session for re-using an HTTP client.
+    :param timeout: An (optional) timeout.
+    :return: A metadata response on successful call or None if there is an error.
+    """
+    # noinspection Mypy
+    handle_resp: Callable[
+        [requests.Response], MetadataRespM
+    ] = lambda resp: MetadataRespM.from_json(resp.json())
+    return post_req(
+        redvox_config,
+        RoutesV1.METADATA_REQ_M,
+        packet_metadata_req,
+        handle_resp,
+        session,
+        timeout,
+    )
+
+
+def request_station_statuses(
+    redvox_config: RedVoxConfig,
+    station_status_req: StationStatusReq,
+    session: Optional[requests.Session] = None,
+    timeout: Optional[float] = None,
+) -> Optional[StationStatusResp]:
+    # noinspection Mypy
+    handle_resp: Callable[
+        [requests.Response], StationStatusResp
+    ] = lambda resp: StationStatusResp.from_dict(resp.json())
+    return post_req(
+        redvox_config,
+        RoutesV1.STATION_STATUS_TIMELINE,
+        station_status_req,
+        handle_resp,
+        session,
+        timeout,
+    )
